/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect to create and populate a table then
fetch and display the data. Connections are aquired through driver manager.

Steps
-----
1. Load the JSQLConnect JDBC Driver.
2. Make a JDBC connection using the JDBC Driver Manager.
3. Create and populate a table.
4. Display the rows from the table.
5. Close the connection.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource, user and password in jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret are valid
3. JSQLConnect.jar is in the classpath.


Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory.

Compile the example at the command line with:-
javac ExampleApplication.java

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleApplication

The results of the result query will be displayed.
*/

import java.sql.*;
import java.util.Properties;

public class ExampleApplication {
  private Connection con;
  private Driver driver;
  private final String TABLE_NAME = "TEST_TABLE";

  public void run() {
    try {
      loadJDBCDriver();
      connectToDatabase();
      populateTable();
      displayTable();
      con.close();
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private boolean loadJDBCDriver() throws SQLException {
    try  {
      driver = (Driver) Class.forName("com.jnetdirect.jsql.JSQLDriver").newInstance();
      log("Loaded driver OK");
      return true;
    }
    catch (Exception e) {
      log("Cannot load the driver, reason:"+e.toString() +
          "Probably the JSQLConnect.jar is not in the classpath");
      throw new SQLException(e.toString());
    }
  }

  private boolean connectToDatabase() throws SQLException {
    String connectStr = "jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret";
    log("Starting connection to "+connectStr);
    try  {
      con = driver.connect(connectStr, new Properties());
      log("Connected to database OK");
      return true;
    }
    catch (SQLException e) {
      log("Cannot connect to database, reason:"+e.toString());
      throw e;
    }
  }

  private void populateTable() throws SQLException {
    log("Creating table ..");
    Statement s = con.createStatement();
    try {
      s.executeUpdate("DROP TABLE "+TABLE_NAME);
    }
    catch (SQLException e) {}

    s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

    log("Populating table rows ..");
    for (int i=0; i<10; i++) {
      s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
    }
  }

  private void displayTable() throws SQLException {
    log("Displaying table rows ..");
    Statement s = con.createStatement();
    ResultSet rs = s.executeQuery("SELECT * FROM "+TABLE_NAME);
    ResultSetMetaData rsMeta = rs.getMetaData();
    int columns = rsMeta.getColumnCount();
    for (int i=0; i<columns; i++)
      log("ColumnName "+i+" "+rsMeta.getColumnName(i+1));
    int row= 0;
    while (rs.next()) {
      String rowData = "";
      for (int i = 0; i<columns; i++) {
        rowData  += rs.getString(i+1)+"   ";
      }
      log("RowData "+ row +":"+rowData );
      row++;
    }
    log("Total rows:"+row);
  }

  private void log(String s) {
    System.out.println("Log:"+s);
  }

  public static void main(String[] args) {
    ExampleApplication app = new ExampleApplication();
    app.run();
  }
}
