/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect to create, update and re-synchronize a table
using a cached rowset. Traversal and updates to the cached rowset are performed while it is
disconnected from the database (offline).

Steps
-----
1. Load the JSQLConnect JDBC Driver.
2. Populate the cached rowset in turn from
  - Connection properties
  - A connected connection
  - A populated result set
3. Traverse and display the rowset.
4. Update the rowset offline.
5. Synchronize the rowset changes to the underlying database table.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource, user and password in jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret are valid

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleCachedRowset

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleCachedRowset

The results of the result query should be displayed.
*/

import java.sql.*;
import java.util.Properties;
import com.jnetdirect.jsql.CachedRowSet;

public class ExampleCachedRowset {
  final String USER_NAME = "sa";
  final String URL = "jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret";
  final String ROWSET_POPULATE_COMMAND = "SELECT * FROM TEST_TABLE";

  final String CHAR_VALUE_ORG = "CHAR_VALUE_ORG";
  java.sql.Date DATE_VALUE_ORG = new java.sql.Date(System.currentTimeMillis());

  final String CHAR_VALUE_UPD = "CHAR_VALUE_UPD";
  final String CHAR_VALUE_INS = "CHAR_VALUE_INS";

  Driver drv;
  int nRows = 5;
  int nDeletes=nRows/2; //rows to delete
  int nInserts=(nRows/2) + 1; //rows to insert

  public static void main(String[] args) {
    ExampleCachedRowset app = new ExampleCachedRowset();
    app.run();
  }

  void run() {
    try {
      loadDriver();
      for (int connectionType=0; connectionType<3; connectionType++) {
        createTableData();

        CachedRowSet cs = null;

        // Populate the cached rowset from the database rows in various ways

        if (connectionType==0)
         cs = populateWithProperties();
        if (connectionType==1)
         cs = populateWithConnection();
        if (connectionType==2)
         cs = populateWithResultSet();

        //At this point the rowset is not connected to the database

        // Display the rowset contents
        displayRowSet(cs, "from database rows");

        //Update the cached rowset rows
        updateRowset(cs);

        //Display the rowset after the updates
        displayRowSet(cs, "after cached updates");

        //Reconnect to the database and synchronize the database rows to the contents of the rowset
        databaseUpdate(cs);

        //Verify the database table was synchronized correctly
        CachedRowSet cs1 = populateWithConnection();
        displayRowSet(cs1, "from database after upate");
      }
    }
    catch (SQLException e) {
      error("SQL Exception:"+e.toString());
    }
  }

  private void updateRowset(CachedRowSet cs) throws SQLException {

    /* Update the cached rowset while it is disconnected from the databse */

    // updates

    cs.beforeFirst();
    int nFound=0;
    int nDelDone=0;
    while (cs.next()) {
      if (nDelDone<nDeletes) { //delete some rows
        cs.deleteRow();
        nDelDone++;
      }
      else { //update some rows
        cs.updateString("COL_CHAR", CHAR_VALUE_UPD);
        cs.updateRow();
      }
      nFound++;
    }

    /* insert some rows */

    for (int i=0; i<nInserts; i++) {
      cs.moveToInsertRow();
      cs.updateInt("COL_INT", i+nRows);
      cs.updateString("COL_CHAR", CHAR_VALUE_INS);
      cs.insertRow();
    }
    cs.moveToCurrentRow();
  }

  private void databaseUpdate(CachedRowSet cs) throws SQLException {

    /* Update the underlying database table with the cached rowset updates */

    Connection con = getConnection();
    cs.acceptChanges(con);
    con.close();
  }

  void createTableData() throws SQLException {
    Connection con = getConnection();
    Statement s = con.createStatement();
    try {
      s.executeUpdate("DROP TABLE TEST_TABLE");
    }
    catch (SQLException e) {}
    s.executeUpdate("CREATE TABLE TEST_TABLE ("+
      "COL_INT INT PRIMARY KEY, "+
      "COL_CHAR CHAR(32), "+
      "COL_VARCHAR CHAR(32))");
    PreparedStatement ps = con.prepareStatement("INSERT INTO TEST_TABLE "+
      "(COL_INT, COL_CHAR, COL_VARCHAR) VALUES "+
      "(?,?,?)");
    for (int i=0; i<nRows; i++) {
      int x=1;
      ps.setInt(x++, i);
      ps.setString(x++, CHAR_VALUE_ORG);
      ps.setString(x++, CHAR_VALUE_ORG);
      ps.executeUpdate();
    }
    con.close();
  }

  private void displayRowSet(CachedRowSet cs, String sContext) throws SQLException {
    ResultSetMetaData md = cs.getMetaData();
    int cols = md.getColumnCount();
    cs.beforeFirst();
    int nRows=0;
    System.out.println("\nRowset contents ("+sContext+") :-");
    System.out.print("Column names ");
    for (int i=0; i<cols; i++) {
      System.out.print(md.getColumnName(i+1)+"  ");
    }
    System.out.println();
    while (cs.next()) {
      for (int i=0; i<cols; i++) {
        System.out.print(cs.getString(i+1)+"  ");
      }
      System.out.println();
      nRows++;
    }
    System.out.println("Row count:"+nRows);
  }

  private CachedRowSet populateWithProperties() throws SQLException {
    System.out.println("\n----- Populating rowset using properties -----");
    CachedRowSet cs = new CachedRowSet();
    cs.setUrl(URL);
    cs.setCommand(ROWSET_POPULATE_COMMAND);
    cs.execute();
    return cs;
  }

  private CachedRowSet populateWithConnection() throws SQLException {
    System.out.println("\n----- Populating rowset using a connection -----");
    CachedRowSet cs = new CachedRowSet();
    Connection c= getConnection();
    cs.setCommand(ROWSET_POPULATE_COMMAND);
    cs.execute(c);
    return cs;
  }

  private CachedRowSet populateWithDataSource() throws SQLException {
    CachedRowSet cs = new CachedRowSet();
    cs.setUsername(USER_NAME);
    cs.setDataSourceName("Databases/Datasource");
    cs.setCommand(ROWSET_POPULATE_COMMAND);
    cs.execute();
    return cs;
  }

  private CachedRowSet populateWithResultSet() throws SQLException {
    System.out.println("\n----- Populating rowset using a result set -----");
    Connection con = getConnection();
    Statement s = con.createStatement();
    ResultSet rs = s.executeQuery(ROWSET_POPULATE_COMMAND);
    CachedRowSet cs = new CachedRowSet();
    cs.populate(rs);
    rs.close();
    s.close();
    con.close();
    return cs;
  }

  private void error(String s) {
    System.out.println(s);
  }

  Connection getConnection() throws SQLException {
    return drv.connect(URL, new Properties());
  }

  void loadDriver() throws SQLException {
    try {
      drv = (Driver) Class.forName("com.jnetdirect.jsql.JSQLDriver").newInstance();
    }
    catch (Exception e) {
      throw new SQLException("Failed to load driver, probably the classpath is not set correctly. "+
        e.toString());
    }
  }
}