/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect to call stored procedures.
The example demonstrates JDBC and database portable techniques to return multiple result sets,
update counts and output parameters from a stored procedure.

Unfortunately, the syntax to define a stored procedure in various database platforms is database specific
(PL-SQL, Transact-SQL etc). Therefore you have to create stored procedures manually in the database before
you can run this example. This example calls two stored procedures named "SIMPLE_PROCEDURE" and "COMPLEX_PROCEDURE".
"SIMPLE_PROCEDURE" returns an integer where as "COMPLEX_PROCEDURE" returns two integers.

However, the JDBC calls to call and process are standard. This enables JDBC code that invokes
stored procedures to be database platform independent. Using standard JDBC calls its also
possible to handle arbitrarily complex stored procedures that return a a combination of result sets,
update counts and returned values. These techniques are illustrated in this example.

Steps
-----
1. Load the JSQLConnect JDBC Driver.
2. Make a JDBC connection using the JDBC Driver Manager.
5. Call the stored procedures.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource, user and password in jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret are valid
3. JSQLConnect.jar is in the classpath.
4. A stored procedure named "SIMPLE_PROCEDURE" which returns an integer.
5. A stored procedure named "COMPLEX_PROCEDURE" which returns two integers.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory.

Compile the example at the command line with:-
javac ExampleCallableProcedure.java

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleCallableProcedure

The results of the result query will be displayed.
*/

import java.sql.*;
import java.util.Properties;

public class ExampleCallableProcedure {
  private Driver driver;
  private final String SIMPLE_PROCEDURE = "SIMPLE_PROCEDURE";
  private final String COMPLEX_PROCEDURE = "COMPLEX_PROCEDURE";

  public void run() {
    try {
      loadJDBCDriver();
      Connection con = connectToDatabase();
      executeSimpleProcedure(con);
      executeComplexProcedure(con);
      con.close();
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private void executeSimpleProcedure(Connection con) throws SQLException {
    /* Prepare the simple procedure call, set an input parameter and return a result set*/
    CallableStatement cs = con.prepareCall("{call "+SIMPLE_PROCEDURE+" ?");
    cs.setInt(1, 5);
    ResultSet rs = cs.executeQuery();
    displayResultSet(rs, "SimpleProcedure");
  }

  private void executeComplexProcedure(Connection con) throws SQLException {
    /* Prepare the procedure call, set the input parameter and process the results */
    CallableStatement cs = con.prepareCall("{? = call "+COMPLEX_PROCEDURE+" ?, ?");
    cs.registerOutParameter(1, Types.INTEGER);
    cs.setInt(2, 5);
    cs.registerOutParameter(3, Types.INTEGER);

    boolean resultSetGenerated = cs.execute();
    int resultSetNumber=0;

    while (true) {
      if (resultSetGenerated) {
        ResultSet rs = cs.getResultSet();
        resultSetNumber++;
        displayResultSet(rs, "ComplexProcedure, Resultset# "+resultSetNumber);
      }
      else {
        int updateCount = cs.getUpdateCount();
        log("update count generated:"+updateCount);
        if (updateCount == -1)
          break; //JDBC Spec: processing complete when there are no more result sets or update counts.
       }
       resultSetGenerated = cs.getMoreResults();
    }
    //JDBC Spec: for maximum portability output values should be obtained after all
    //results are processed.
    log("return value:"+cs.getInt(1));
    log("output value:"+cs.getInt(3));
  }

  private void displayResultSet(ResultSet rs, String context) throws SQLException {
    ResultSetMetaData rsMeta = rs.getMetaData();
    int columns = rsMeta.getColumnCount();
    log("------ Begin resultSet display ("+context+")------");
    for (int i=0; i<columns; i++)
      log("ColumnName "+i+" "+rsMeta.getColumnName(i+1));
    int row=0;
    while (rs.next()) {
      String rowData = "";
      for (int i = 0; i<columns; i++) {
        rowData  += rs.getString(i+1)+"   ";
      }
      log("RowData "+ row +":"+rowData );
      row++;
    }
    log("------ End resultSet display ------");
  }

  private boolean loadJDBCDriver() throws SQLException {
    try  {
      driver = (Driver) Class.forName("com.jnetdirect.jsql.JSQLDriver").newInstance();
      log("Loaded driver OK");
      return true;
    }
    catch (Exception e) {
      log("Cannot load the driver, reason:"+e.toString() +
          "Probably the JSQLConnect.jar is not in the classpath");
      throw new SQLException(e.toString());
    }
  }

  private Connection connectToDatabase() throws SQLException {
    Connection con = null;
    String connectStr = "jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret";
    log("Starting connection to "+connectStr);
    try  {
      con = driver.connect(connectStr, new Properties());
      log("Connected to database OK");
      return con;
    }
    catch (SQLException e) {
      log("Cannot connect to database, reason:"+e.toString());
      throw e;
    }
  }

  private void log(String s) {
    System.out.println(s);
  }

  public static void main(String[] args) {
    new ExampleCallableProcedure().run();
  }
}
