/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using connection pooling using a JSQLConnect
connection pooling data source. The datasource is dynamically instanced in this example
but can also be bound and instanced from JNDI if required.

Steps
-----
1. Dynamically create a connection pooling datasource.
2. Use the datasource to populate and display a table.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource, user and password set in the datasource are valid.
3. JSQLConnect.jar is in the classpath.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory. Ensure any required JNDI jars are also in the classpath.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleConnectionPool.java

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleConnectionPool

The status of the connection pool should be displayed.
*/

import java.io.*;
import java.sql.*;
import javax.sql.*;
import javax.naming.*;
import java.util.*;
import com.jnetdirect.jsql.JSQLPoolingDataSource;

public class ExampleConnectionPool {
  private final String TABLE_NAME = "TEST_TABLE";

  public static void main(String[] args) {
    new ExampleConnectionPool().runExample();
  }

  void runExample() {
    try {
      JSQLPoolingDataSource poolingDS = makePoolingDataSource();
      useConnectionPool(poolingDS);
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private JSQLPoolingDataSource makePoolingDataSource() {
    /* Create the datasource to provide pooled connections. */
    JSQLPoolingDataSource ds = new JSQLPoolingDataSource();
    ds.setServerName("localhost");
    ds.setDatabaseName("master");
    ds.setUser("sa");
    ds.setPassword("secret");
    ds.setApplicationName("ExampleApplication");
    //Any other datasource specific properties can be set here..

    /* Configure some connection pool properties */
    ds.setMinPoolSize(2);
    ds.setMaxPoolSize(10);
    ds.setMaxIdleTime(60);
    ds.setManagementCycleTime(2);

    return ds;
  }

  private void useConnectionPool(DataSource pooledDS) throws SQLException {
    int nExpectedRows = -1;

    // Get multiple connections from the connection pool and use them.
    // Two physical and sixty logical connections will be created by this loop.
    for (int i = 0; i<30; i++) {
      Connection c0 = pooledDS.getConnection();
      Connection c1 = pooledDS.getConnection();

      Statement s = c0.createStatement();
      ResultSet rs = s.executeQuery("SELECT * FROM " + TABLE_NAME);
      int nRows = 0;
      while (rs.next()) {
        String sx = rs.getString(1);
        nRows++;
      }
      if (nExpectedRows < 0)
        nExpectedRows = nRows;
      else {
        if (nRows != nExpectedRows) {
          throw new SQLException("Wrong row count, got:"+nRows+" expected:"+nExpectedRows);
        }
      }
      c0.close();
      c1.close();
    }

    // Ask the pool manager to display its status
    System.out.println(((JSQLPoolingDataSource) pooledDS).getStatus());
  }

  void log(String s){
    System.out.println(s);
  }
}