/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect using a JDBC dynamically created
datasource. This example shows how a datasource can be used without using JNDI.

Steps
-----
1. Dynamically create a datasource.
2. Use the datasource to populate and display a table.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource database (master), user (sa)
   and password(secret) are valid.
3. JSQLConnect.jar is in the classpath.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory. Ensure any required JNDI jars are also in the classpath.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleDataSource.java

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleDataSource

The results of the result query will be displayed.
*/

import java.io.*;
import java.sql.*;
import javax.sql.*;
import java.util.*;
import com.jnetdirect.jsql.JSQLDataSource;

public class ExampleDataSource {
  private final String TABLE_NAME = "TEST_TABLE";

  public static void main(String[] args) {
    ExampleDataSource app = new ExampleDataSource();
    app.run();
  }

  private void run() {
    try {
      DataSource ds = makeDataSource();
      populateTable(ds);
      showTable(ds);
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private DataSource makeDataSource() {
    /* Create the data source dynamically */
    JSQLDataSource ds = new JSQLDataSource();
    ds.setServerName("localhost");
    ds.setDatabase("master");
    ds.setUser("sa");
    ds.setPassword("secret");
    ds.setApplicationName("ExampleApplication");
    //Any other datasource specific properties can be set here..
    return ds;
  }

  private void populateTable(DataSource ds) throws SQLException {
    Connection con = ds.getConnection();
    log("Creating table ..");
    Statement s = con.createStatement();
    try {
      s.executeUpdate("DROP TABLE "+TABLE_NAME);
    }
    catch (SQLException e) {}

    s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

    log("Populating table rows ..");
    for (int i=0; i<10; i++) {
      s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
    }
    con.close();
  }

  private void showTable(DataSource ds) throws SQLException {
    Connection con = ds.getConnection();
    Statement s = con.createStatement();
    ResultSet rs = s.executeQuery("SELECT * FROM " + TABLE_NAME);
    ResultSetMetaData dbMeta = rs.getMetaData();
    int columns = dbMeta.getColumnCount();
    int rows = 0;

    while (rs.next()) {
      String rowData = "";
      for (int i = 0; i<columns; i++) {
        rowData += rs.getString(i+1)+"   ";
      }
      System.out.println("ROW Data:"+rowData);
      rows++;
    }
    con.close();
    log("Displayed "+rows+" rows");
  }


  void log(String s){
    System.out.println(s);
  }
}