/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect using a JDBC
datasource bound to and instanced from JNDI.

Steps
-----
1. Dynamically create a datasource and bind it to a JNDI context.
2. Lookup the datasource in JNDI and use it to populate and display a table.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource database (master), user (sa)
   and password(secret) are valid.
3. You have installed the JNDI API download from www.javasoft.com if not using JDK 1.4 or higher.
4. You have installed the sample JNDI file store provider.
   (http://www.javasoft.com/products/jndi/index.html)
5. JSQLConnect.jar is the in the classpath.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory. Ensure any required JNDI jars are also in the classpath.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleDataSourceJNDI.java

Run the example at the command line. Also ensure the file system provider
(fscontext.jar,providerutil.jar etc) is in the classpath.

java -classpath .;./JSQLConnect.jar ExampleDataSourceJNDI

The results of the result query will be displayed.
*/

import java.io.*;
import java.sql.*;
import javax.sql.*;
import javax.naming.*;
import java.util.*;
import com.jnetdirect.jsql.JSQLDataSource;

public class ExampleDataSourceJNDI {
  private String sFileStore = "TestJNDIFileStore"; // The name of the file for the JNDI implementation
  private String sDSName = "MyDataSource"; //The name of the datasource
  private Context ctx; // JNDI context
  private final String TABLE_NAME = "TEST_TABLE";

  public static void main(String[] args) {
    ExampleDataSourceJNDI app = new ExampleDataSourceJNDI();
    app.run();
  }

  private void run() {
    try {
      /* Example using JNDI */
      initJNDI();
      registerDataSource();
      DataSource ds = getJNDIDataSource();
      populateTable(ds);
      showTable(ds);
      //useJNDIDataSource();
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private DataSource makeDynamicDataSource() {
    /* Create the data source dynamically */
    JSQLDataSource ds = new JSQLDataSource();
    ds.setServerName("localhost");
    ds.setDatabase("master");
    ds.setUser("sa");
    ds.setPassword("secret");
    ds.setApplicationName("ExampleApplication");
    //Any other datasource specific properties can be set here..
    return ds;
  }

  private void registerDataSource() throws SQLException {
    /* Create a data source and register it in JNDI. This is typically performed by the system
       administrator as your application is installed. */
    try {
      DataSource ds = makeDynamicDataSource();
      ctx.rebind(sDSName, ds);
    }
    catch (Exception e) {
      throw new SQLException(e.toString());
    }
  }

  private DataSource getJNDIDataSource() throws SQLException {
    /* Retrieve the bound data source from JNDI */
    try {
      Context ctx = new InitialContext(System.getProperties());
      DataSource ds = (DataSource) ctx.lookup(sDSName);
      return ds;
    }
    catch (NamingException e) {
      throw new SQLException(e.toString());
    }
  }

  private void populateTable(DataSource ds) throws SQLException {
    Connection con = ds.getConnection();
    log("Creating table ..");
    Statement s = con.createStatement();
    try {
      s.executeUpdate("DROP TABLE "+TABLE_NAME);
    }
    catch (SQLException e) {}

    s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

    log("Populating table rows ..");
    for (int i=0; i<10; i++) {
      s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
    }
    con.close();
  }

  private void showTable(DataSource ds) throws SQLException {
    Connection con = ds.getConnection();
    Statement s = con.createStatement();
    ResultSet rs = s.executeQuery("SELECT * FROM " + TABLE_NAME);
    ResultSetMetaData dbMeta = rs.getMetaData();
    int columns = dbMeta.getColumnCount();
    int rows = 0;

    while (rs.next()) {
      String rowData = "";
      for (int i = 0; i<columns; i++) {
        rowData += rs.getString(i+1)+"   ";
      }
      System.out.println("ROW Data:"+rowData);
      rows++;
    }
    con.close();
    log("Displayed "+rows+" rows");
  }

  private void initJNDI() {

    /* Create the file store first time - the file store holds the JNDI entries */
    File root = new File(sFileStore);
    if (!root.exists())
      root.mkdir();

    /* Initialize JNDI - tell it to use the File Store provider */
    Hashtable env;
    env = System.getProperties();
    env.put(Context.INITIAL_CONTEXT_FACTORY,
      "com.sun.jndi.fscontext.RefFSContextFactory");
    env.put(Context.PROVIDER_URL, "file:"+sFileStore+"/");

    /* Create a JNDI context and bind it in the naming service */
    try {
      ctx = new InitialContext(env);
    }
    catch (javax.naming.NamingException e) {
      e.printStackTrace();
    }
  }

  void log(String s){
    System.out.println(s);
  }
}