/* ***** JSQLConnect Examples ******

Overview
--------
This sample application demonstrates using JSQLConnect to create and navigate a JDBC rowset.

Steps
-----
1. Create and populate a table.
2. Create a JDBC rowset using the table.
3. Create and register a rowset listener.
4. Navigate the rowset and have the listener methods invoked.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource, user and password in jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret are valid.
3. JSQLConnect.jar is in the classpath.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleJDBCRowset

Run the example at the command line with:-
java -classpath .;./JSQLConnect.jar ExampleJDBCRowset

The results of the result query should be displayed.
*/

import javax.sql.*;
import java.sql.*;
import com.jnetdirect.jsql.JSQLJDBCRowset;
import com.jnetdirect.jsql.JSQLDataSource;

class RowsetListener implements RowSetListener {
  public void cursorMoved(RowSetEvent event)  {
    System.out.println("Listener:Moved");
  }
  public void rowChanged(RowSetEvent event) {
    System.out.println("Listener:RowChanged");
  }
  public void rowSetChanged(RowSetEvent event){
    System.out.println("Listener:SetChanged");
  }
}

public class ExampleJDBCRowset {
  private final String TABLE_NAME = "TEST_TABLE";

  public static void main(String[] args) {
    new ExampleJDBCRowset().run();
  }

  private void run() {
    try {
      populateTable();

      JSQLJDBCRowset jrs = new JSQLJDBCRowset();
      jrs.setUrl("jdbc:JSQLConnect://localhost/database=master");
      jrs.setUsername("sa");
      jrs.setPassword("secret");
      jrs.setCommand("SELECT * FROM " +TABLE_NAME);

      RowsetListener l = new RowsetListener();
      jrs.addRowSetListener(l);

      jrs.execute();
      while (jrs.next()) {
        System.out.println(jrs.getString(1));
      }
      jrs.close();
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  private void populateTable() throws SQLException {
    JSQLDataSource ds = new JSQLDataSource();
    ds.setServerName("localhost");
    ds.setDatabase("master");
    ds.setUser("sa");
    ds.setPassword("secret");
    log("Creating table ..");

    Connection con = ds.getConnection();
    Statement s = con.createStatement();
    try {
      s.executeUpdate("DROP TABLE "+TABLE_NAME);
    }
    catch (SQLException e) {}

    s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

    log("Populating table rows ..");
    for (int i=0; i<10; i++) {
      s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
    }
  }
  private void log(String s) {
    System.out.println(s);
  }
}