/* ***** JSQLConnect Examples ******

Overview
--------
This sample servlet demonstrates using JSQLConnect to create and populate a table then
fetch and display the data. Database connections are aquired from the JDBC driver manager.

Steps
-----
1. Load the JSQLConnect JDBC Driver during the servlet init
2. Make a JDBC connection using the JDBC Driver Manager during the servlet init.
3. Create and populate a table during the servlet init.
4. Display the rows from the table during the servlet GET.
5. Close the connection.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource database (master), user (sa)
   and password(secret) are valid.
3. JSQLConnect.jar is in the classpath of your servlet engine.
   For example, if you are using Apache Tomcat 4.x JSQLConnect.jar should be in %CATALINA_HOME%\common\lib
   If you are using another servlet engine please consult that product's documentation regarding
   deployment of 3rd party libraries.
4. Your servlet engine and web server are running.

Compile & Run
-------------
Compile the example at the command line with:-
javac ExampleServlet

The specific procedure to deploy the example depends on the servlet engine you are using.
Under Tomcat you can deploy the example by copying ExampleServlet.class to
%CATALINA_HOME%/webapps/examples/WEB-INF/classes

The specific URL to run the example depends on the servlet engine you are using.
Under Tomcat you can run the example at a web browser with:-
http://localhost:8080/examples/servlet/ExampleServlet

The results of the result query will be displayed.
*/
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.Properties;

public class ExampleServlet extends HttpServlet {
  private Connection con;
  private Driver driver;
  private String lastErr;
  private final String TABLE_NAME = "TEST_TABLE";

  public void init(ServletConfig c) throws ServletException {
    /* Load the driver */
    try  {
      driver = (Driver) Class.forName("com.jnetdirect.jsql.JSQLDriver").newInstance();
    }
    catch (Exception e) {
        lastErr = "Cannot load the driver, reason:"+e.toString() +
            "Probably JSQLConnect.jar is not in the servlet engine classpath";
        return;
    }

    /* Connect to the database */

    String connectStr = "jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret";
    logMsg("Starting connection to "+connectStr);
    try  {
      con = driver.connect(connectStr, new Properties());
    }
    catch (SQLException e) {
      lastErr = "Cannot connect to "+connectStr+", reason:"+e.toString();
    }
    populateTable();
  }

  private void populateTable()  {
    logMsg("Creating table ..");
    try {
      Statement s = con.createStatement();
      try {
        s.executeUpdate("DROP TABLE "+TABLE_NAME);
      }
      catch (SQLException e) {}

      s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

      logMsg("Populating table rows ..");
      for (int i=0; i<10; i++) {
        s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
      }
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  public void doGet (HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException {

    PrintWriter	out;
    res.setContentType("text/html");
    out = res.getWriter();
    String sHead = "JSQLConnect ExampleServlet";
    out.println("<html>");
    out.println("<head><title>"+sHead+"</title></head>");
    out.println("<body>");
    out.println("<hr><font size='5'>"+sHead+"</font><hr>");
    out.println("DriverClass:com.jnetdirect.jsql.JSQLDriver<br>");
    out.println("Driver URL:jdbc:JSQLConnect://localhost/database=master/user=sa/password=secret<br>");
    out.println("<br>Table rows display:-<br>");

    int nRows = 0;

    if (lastErr != null) {
      out.println(lastErr);
    }
    else {
      try {
        Statement s = con.createStatement();
        ResultSet rs = s.executeQuery("SELECT * FROM "+TABLE_NAME);
        ResultSetMetaData dbMeta = rs.getMetaData();
        int nColumns = dbMeta.getColumnCount();
        while (rs.next()) {
          String sRow = "";
          for (int i = 0; i<nColumns; i++) {
            sRow += rs.getString(i+1)+"   ";
          }
          out.println(sRow + "<br>");
          nRows++;
        }
      }
      catch (SQLException e) {
        out.println("SQL Error, reason:"+e.toString());
      }
    }
    out.println("Rows:"+nRows+"<hr>");
    out.println("</body></html>");
  }

  public String getServletInfo() {
    return "ExampleServlet - JSQLConnect";
  }


  public void destroy() {
    if (con != null) {
      try {
        con.close();
      }
      catch (SQLException e) {
        logMsg("Failed to close connection:"+e.toString());
      }
    }
  }

  private void logMsg(String s) {
    System.out.println(s);
  }

}

