/* ***** JSQLConnect Examples ******

Overview
--------
This sample servlet demonstrates using JSQLConnect to create and populate a table then
fetch and display the data. Database connections are aquired from a datasource
configured by the servlet engine.

Steps
-----
1. Lookup the datasource in the JNDI context.
2. Acquire the JDBC connection from the datasource.
3. Create and populate a table during the servlet init.
4. Display the rows from the table during the servlet GET.
5. Close the connection.

Requires
--------
1. Microsoft SQL Server is up and running.

2. JSQLConnect.jar is in the classpath of your servlet engine.
   For example, if you are using Apache Tomcat 4.x JSQLConnect.jar should be in %CATALINA_HOME%\common\lib
   If you are using another servlet engine please consult that product's documentation regarding
   deployment of 3rd party libraries.

3. You have configured the correct resource references and JNDI datasources in your
   servlet engine's configuration.

For example, if you are using Apache Tomcat 4.x:-

a) configure the JNDI resource in $CATALINA_HOME/conf/server.xml file:
<Resource name="jdbc/JSQLDataSource" auth="SERVLET"
  type="javax.sql.DataSource"/>
<ResourceParams name="jdbc/JSQLDataSource">
  <parameter><name>user</name><value>sa</value></parameter>
  <parameter><name>password</name><value>secret</value></parameter>
  <parameter><name>driverClassName</name>
  <value>com.jnetdirect.jsql.JSQLDriver</value></parameter>
  <parameter><name>driverName</name>
  <value>jdbc:JSQLConnect://localhost/database=master</value></parameter>
</ResourceParams>

b) configure the resource-ref in the application's web.xml file:-
<resource-ref>
  <res-ref-name>jdbc/JSQLDataSource</res-ref-name>
  <res-type>javax.sql.DataSource</res-type>
  <res-auth>Container</res-auth>
</resource-ref>

4. Your servlet engine and web server are running.

Compile & Run
-------------
Compile the example at the command line with:-
javac ExampleServletJNDI

The specific procedure to deploy the example depends on the servlet engine you are using.
Under Tomcat you can deploy the example by copying ExampleServletJNDI.class to
%CATALINA_HOME%/webapps/examples/WEB-INF/classes

The specific URL to run the example depends on the servlet engine you are using.
Under Tomcat you can run the example at a web browser with:-
http://localhost:8080/examples/servlet/ExampleServletJNDI

The results of the result query will be displayed.
*/
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.io.*;
import java.net.*;
import java.sql.*;
import javax.sql.*;
import java.util.Properties;
import javax.naming.*;
import java.util.*;

public class ExampleServletJNDI extends HttpServlet {
  private Driver driver;
  private String lastErr;
  private final String TABLE_NAME = "TEST_TABLE";
  private DataSource ds = null;

  public void init(ServletConfig c) throws ServletException {
    try {

      // Look up the JNDI data source
      Context initCtx = new InitialContext();
      Context envCtx = (Context) initCtx.lookup("java:comp/env");
      ds = (DataSource) envCtx.lookup("jdbc/JSQLDataSource");

      // Acquire the connection
      Connection con = ds.getConnection();
      populateTable(con);
    }
    catch (NamingException e) {
      throw new ServletException(e.toString());
    }
    catch (SQLException sqle) {
      throw new ServletException(sqle.toString());
    }
  }

  private void populateTable(Connection con)  {
    logMsg("Creating table ..");
    try {
      Statement s = con.createStatement();
      try {
        s.executeUpdate("DROP TABLE "+TABLE_NAME);
      }
      catch (SQLException e) {}

      s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

      logMsg("Populating table rows ..");
      for (int i=0; i<10; i++) {
        s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
      }
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  public void doGet (HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException {

    PrintWriter	out;
    res.setContentType("text/html");
    out = res.getWriter();
    String sHead = "JSQLConnect ExampleServletJNDI";
    out.println("<html>");
    out.println("<head><title>"+sHead+"</title></head>");
    out.println("<body>");
    out.println("<hr><font size='5'>"+sHead+"</font><hr>");
    out.println("DataSource:jdbc/JSQLDataSource<br>");
    out.println("<br>Table rows display:-<br>");

    int nRows = 0;
    Connection con=null;
    if (lastErr != null) {
      out.println(lastErr);
    }
    else {
      try {
        con = ds.getConnection();
        Statement s = con.createStatement();
        ResultSet rs = s.executeQuery("SELECT * FROM "+TABLE_NAME);
        ResultSetMetaData dbMeta = rs.getMetaData();
        int nColumns = dbMeta.getColumnCount();
        while (rs.next()) {
          String sRow = "";
          for (int i = 0; i<nColumns; i++) {
            sRow += rs.getString(i+1)+"   ";
          }
          out.println(sRow + "<br>");
          nRows++;
        }
        con.close();
      }
      catch (SQLException e) {
        out.println("SQL Error, reason:"+e.toString());
      }
    }
    out.println("Rows:"+nRows+"<hr>");
    out.println("</body></html>");
  }

  public String getServletInfo() {
    return "ExampleServlet - JSQLConnect";
  }

  private void logMsg(String s) {
    System.out.println(s);
  }

}

