/* ***** JSQLConnect Examples ******

Overview
--------
This example servlet demonstrates using JSQLConnect to pool database connections in a servlet.
This example creates the pooling datasource dynamically. In this example
the pooling is managed by the JSQLPoolingDataSource .

In practice the database name, user, password etc would not be hard coded. They are hard coded
here only to simplify the example code.

Alternativly, your servlet/JSP engine may support one or either of:-

1. The JSQLPoolingDataSource datasource may be bound to a JNDI datasource within the servlet
engine's datasources configuration. In this cases the pooling is still managed by the JSQLConnect
driver and you may specify values for any JSQLPoolingDataSource property.

2. You can create a connection pool managed by the servlet engine. Such a configuration would
use the JSQLConnect driver but the pool would be managed by the servlet engine.

Check your servlet engine's datasource configuration documentation for more details.The pooling
datasource is setup in the servlet init(). This ensures that the JNDI lookup
(which is typically slow) and datasource creation are only performed at servlet initialization
and not performed for every client invocation of the servlet.

When a client uses the servlet the pool only creates a physical connection if one is not already
available. During client usage of the servlet no JNDI lookups are performed.

Steps
-----
1. Creates the pooling datasource in the servlet init
2. Create and populate a table during the servlet init.
3. Display the rows from the table during the servlet GET.
4. Display the status of the connection pool.
5. Close the connection.

Requires
--------
1. Microsoft SQL Server is up and running.
2. The datasource database (master), user (sa)
   and password(secret) are valid.
3. JSQLConnect.jar is in the classpath of your servlet engine.
   For example, if you are using Apache Tomcat 4.x JSQLConnect.jar should be in %CATALINA_HOME%\common\lib
   If you are using another servlet engine please consult that product's documentation regarding
   deployment of 3rd party libraries.
4. Your servlet engine and web server are running.

Compile & Run
-------------
Locate JSQLConnect.jar in the JSQLConnect download or distribution and copy it to this
examples directory.

Compile the example at the command line with:-
javac -classpath ./JSQLConnect.jar ExampleServletWithPooling

The specific procedure to deploy the example depends on the servlet engine you are using.
Under Tomcat you can deploy the example by copying ExampleServletWithPooling.class to
%CATALINA_HOME%/webapps/examples/WEB-INF/classes

The specific URL to run the example depends on the servlet engine you are using.
Under Tomcat you can run the example at a web browser with:-
http://localhost:8080/examples/servlet/ExampleServletWithPooling

The results of the result query will be displayed.
*/
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.Properties;
import com.jnetdirect.jsql.JSQLPoolingDataSource;

public class ExampleServletWithPooling extends HttpServlet {
  private final String TABLE_NAME = "TEST_TABLE";
  private JSQLPoolingDataSource poolingDataSource;

  public void init(ServletConfig c) throws ServletException {
    try {
      poolingDataSource = new JSQLPoolingDataSource();
      poolingDataSource.setServerName("localhost");
      poolingDataSource.setDatabase("master");
      poolingDataSource.setUser("sa");
      poolingDataSource.setPassword("secret");
      poolingDataSource.setApplicationName("ExampleApplication");
      //Any other datasource specific properties can be set here..

      Connection con = poolingDataSource.getConnection();
      populateTable(con);
      con.close(); //return the connection to the pool
    } catch (SQLException e) {
      throw new ServletException(e.toString());
    }
  }

  private void populateTable(Connection con)  {
    logMsg("Creating table ..");
    try {
      Statement s = con.createStatement();
      try {
        s.executeUpdate("DROP TABLE "+TABLE_NAME);
      }
      catch (SQLException e) {}

      s.executeUpdate("CREATE TABLE "+TABLE_NAME+" (COL1 INTEGER, COL2 VARCHAR(10))");

      logMsg("Populating table rows ..");
      for (int i=0; i<10; i++) {
        s.executeUpdate("INSERT INTO "+TABLE_NAME+" (COL1, COL2) VALUES ("+i+", 'VALUE"+ i+"')");
      }
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
  }

  public void doGet (HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException {

    PrintWriter	out;
    res.setContentType("text/html");
    out = res.getWriter();
    String sHead = "JSQLConnect ExampleServlet With Connection Pooling";
    out.println("<html>");
    out.println("<head><title>"+sHead+"</title></head>");
    out.println("<body>");
    out.println("<hr><font size='5'>"+sHead+"</font><hr>");
    out.println("DataSource:JSQLPoolingDataSource<br>");
    out.println("<br>Table rows display:-<br>");

    int nRows = 0;
    Connection con = null;

    try {
      con = poolingDataSource.getConnection();
      Statement s = con.createStatement();
      ResultSet rs = s.executeQuery("SELECT * FROM "+TABLE_NAME);
      ResultSetMetaData dbMeta = rs.getMetaData();
      int nColumns = dbMeta.getColumnCount();
      while (rs.next()) {
        String sRow = "";
        for (int i = 0; i<nColumns; i++) {
          sRow += rs.getString(i+1)+"   ";
        }
        out.println(sRow + "<br>");
        nRows++;
      }
    }
    catch (SQLException e) {
      out.println("SQL Error, reason:"+e.toString());
    }
    finally {
      try {
        con.close();
      } catch (SQLException e) {}
    }

    out.println("Rows:"+nRows+"<hr>");
    out.println("<b>Pool Status:</b><br>"+poolingDataSource.getStatus()+"<hr>");
    out.println("</body></html>");
  }

  public String getServletInfo() {
    return "ExampleServlet - JSQLConnect";
  }

  private void logMsg(String s) {
    System.out.println(s);
  }

}

